#ifndef _KDEMASTEREDITOR_C_
#define _KDEMASTEREDITOR_C_

#include <iostream.h>

#include "kdeMasterEditor.h"
#include "part.h"
#include "track.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qscrollview.h>
#include "math.h"
#include "masterEvent.h"
#include "position.h"
#include "prPartEditor.h"
#include "iterator.h"
#include "removeElement.h"
#include "addElement.h"
#include "kdeEditorToolBar.h"
#include "kdeEditorScrollBar.h"
#include "brahmsResources.h"
#include "kdeEditorMenuBar.h"
#include "kdeMasterContent.h"

#include "event.h"
#include "song.h"

extern Song * sonG;

KdeMasterEditor::KdeMasterEditor(Part * p) : KMainWindow(0L, "MasterEventEditor"), PrPartEditor(MASTEREDITOR), index(-1), time1(4), time2(2), newPos(0), newTempo(120) {

  setCaption("Master Event Editor"); // TODO: part name
  _toolbarstate = true;
  _sliderstate = true;

  setMinimumWidth(720);
  setMinimumHeight(360);

  //
  // Toolbars:
  // =========

  _toolbar = new KdeEditorToolBar(this,this);
  _scrollbar = new KdeEditorScrollBar(this,this);

  addToolBar( _toolbar ); _toolbar->setBarPos(KToolBar::Top); _toolbar->show();
  addToolBar( _scrollbar ); _scrollbar->setBarPos(KToolBar::Bottom); if (_sliderstate) _scrollbar->show(); else _scrollbar->hide();

  //
  // Menubar:
  // ========
  //

  _filemenu = new QPopupMenu( this );
  _editmenu = new QPopupMenu( this );
  _viewmenu = new QPopupMenu( this );
  KdeEditorMenuBar::createMenuBar(menuBar(), _filemenu, _editmenu, 0, 0, _viewmenu);
  _viewmenu->setItemChecked(ID_VIEW_TOOLBAR, _toolbarstate);
  _viewmenu->setItemChecked(ID_VIEW_NOTEBAR , false);
  _viewmenu->setItemChecked(ID_VIEW_BUTTONBAR, false);
  _viewmenu->setItemChecked(ID_VIEW_SLIDER, _sliderstate);
  connect(_filemenu,SIGNAL(activated(int)),SLOT(slotMenu(int)));
  connect(_editmenu,SIGNAL(activated(int)),SLOT(slotMenu(int)));
  connect(_viewmenu,SIGNAL(activated(int)),SLOT(slotViewMenu(int)));

  _viewmenu->setItemEnabled(ID_VIEW_NOTEBAR, false);
  _viewmenu->setItemEnabled(ID_VIEW_BUTTONBAR, false);



  //
  // Contents:
  // =========
  //

  QWidget * _main = new QWidget( this, "main widget" );
  setCentralWidget( _main );
  _main->show();


  QLabel * lPos = new QLabel("position",_main);
  lPos->setGeometry(2,4,64,18);

  QLabel * lTempo = new QLabel("tempo",_main);
  lTempo->setGeometry(68,4,64,18);

  QLabel * lMeter = new QLabel("meter",_main);
  lMeter->setGeometry(124,4,64,18);

  posline = new QLineEdit(_main);
  posline->setGeometry(2,24,64,18);
  posline->show();

  templine = new QLineEdit(_main);
  templine->setGeometry(68,24,64,18);
  templine->show();

  metline0 = new QLineEdit(_main);
  metline0->setGeometry(124,24,64,18);
  metline0->show();

  /*  metline1 = new QLineEdit(_main);
  metline1->setGeometry(124,56,30,18);
  metline1->show();
  */
  QPushButton * addTempoBtn = new QPushButton("set tempo",_main);
  addTempoBtn->setGeometry(2,44,92,20);
  addTempoBtn->show();
  connect(addTempoBtn,SIGNAL(released()),SLOT(addTempo()));

  QPushButton * addMeterBtn = new QPushButton("set meter",_main);
  addMeterBtn->setGeometry(96,44,92,20);
  addMeterBtn->show();
  connect(addMeterBtn,SIGNAL(released()),SLOT(addMeter()));

  delBtn = new QPushButton("remove",_main);
  delBtn->setGeometry(2,70,48,48);
  delBtn->show();
  connect(delBtn,SIGNAL(released()),SLOT(remove()));
  delBtn->setEnabled(false);

  QPushButton * exit = new QPushButton(" OK ",_main);
  exit->setGeometry(140,70,48,48);
  exit->show();
  connect(exit,SIGNAL(released()),SLOT(exit()));


  QLabel * title = new QLabel("Mastertrack",_main);
  title->setGeometry(2,122,186,22);

  view = new QListBox(_main);
  view->setGeometry(2,144,186,134);
  view->show();
  connect(view,SIGNAL(highlighted(int)),SLOT(enableDelete(int)));


  //
  // graphical editor
  //

  _content = new KdeMasterContent( _main, this, _toolbar );
  _content->setFrameStyle( QFrame::Sunken );
  _content->setGeometry( 200, 24, 514, 252 );
  _content->show();

  /*  _scroller = new QScrollView( _main );
  _scroller->setFrameStyle( QFrame::Sunken );
  _scroller->setVScrollBarMode( QScrollView::AlwaysOff );
  _scroller->setHScrollBarMode( QScrollView::Auto );
  // _scroller->verticalScrollBar()->setLineStep( _track_height );
  // _scroller->verticalScrollBar()->setPageStep( _main->height() );
  _content = new KdeMasterContent( _scroller->viewport(), this, _toolbar );
  _content->setFrameStyle( QFrame::Sunken );
  _scroller->addChild( _content );
  _content->setGeometry( 0, 0, 2500, 272 );
  // connect( scroller->horizontalScrollBar(), SIGNAL( valueChanged( int ) ), SLOT( slotRightScroll( int ) ) );
  // connect( scroller->verticalScrollBar(), SIGNAL( valueChanged( int ) ), SLOT( slotVerticalScroll( int ) ) );
  _scroller->setGeometry(200, 24, 514, 272);
  _content->show();
  _scroller->show();*/

  setCentralWidget(_main);
  showView();
  update();
  sonG->updateGui();
}

void KdeMasterEditor::resizeEvent( QResizeEvent * )
{
  int h = height() - 60 - 20;
  int w = width() - 20;
  view->setGeometry(2, 144, 186, h-148);
  // _scroller->setGeometry(200, 24, 514, h-28);
  _content->setGeometry(200, 24, w-220, h-28);
}

void KdeMasterEditor::enableDelete(int i) {
  index = i;
  delBtn->setEnabled(true);
  int k = 0;
  Event * ev = 0;
  Part * _part = part();

  for (Iterator i = Iterator(_part); !i.done() && k<=index; i++) {
    ev = (Event*) *i;
    k++;
  }

  if (ev!=0 && ev->isA()==MASTEREVENT) {
    MasterEvent * te = (MasterEvent*) ev;
    char * x = new char[40];
    Position pos = te->internalStart();
    // sprintf(x,"%d. %d. %d\0",pos.gBar(part),pos.gBeat(part),pos.gTicks(part));
    int a = 0; int b = 0; int c = 0;
    sonG->bbt(a,b,c,pos);
    snprintf(x,30," %d. %d. %d",a,b,c);
    posline->setText(x);
    if (te->tempo()>0) { sprintf(x,"%d",te->tempo()); templine->setText(x); metline0->setText(""); }
    else { sprintf(x,"%d / %d",te->meter0(),te->meter1()); metline0->setText(x); templine->setText(""); }
    delete[] x;
  } else delBtn->setEnabled(false);
}

KdeMasterEditor::~KdeMasterEditor() {
  delete view;
  delete posline;
  delete templine;
  delete metline0;

}

void KdeMasterEditor::pos() {
  posline->clearFocus();
  const char * line = posline->text();
  int x1 = atoi(line);
  for (;line[0]!='.';line++) {} line++; for (;line[0]==' ';line++) {}
  int x2 = atoi(line);
  for (;line[0]!='.';line++) {} line++; for (;line[0]==' ';line++) {}
  int x3 = atoi(line);
  // x1 . x2 . x3

  newPos = Position(x1,x2,x3);

  // int ticks = int(1536.0*time1/pow(2,time2));
  // newPos = ticks*(x1-1) + ticks*(x2-1)/time1 + x3;
  //posline->setText("");
}

void KdeMasterEditor::temp() {
  templine->clearFocus();
  newTempo = atoi(templine->text());
  templine->setText("");
}

void KdeMasterEditor::meter() {
  metline0->clearFocus();
  const char * mtxt = metline0->text();
  newMeter0 = atoi(mtxt);
  int i = 0;
  for (;(mtxt[0]!='/')&&(i<10);i++) mtxt++;
  mtxt++;
  newMeter1 = atoi(mtxt);
  if (newMeter1<1) newMeter1 = 4;
  metline0->setText("");
}

void KdeMasterEditor::addTempo() {
  pos();
  temp();
  Part * _part = part();
  if ((newPos>=0)&&(newTempo>0)) {
    if (index!=-1) {
      int k = 0;
      Event * ev = 0;
      for (Iterator i = Iterator(_part); !i.done() && k<=index; i++) {
	ev = (Event*) *i;
	k++;
      }
      if (ev!=0) sonG->doo(new RemoveElement(ev, _part));
    }
    MasterEvent * newTE = new MasterEvent(newPos,newTempo);
    sonG->doo(new AddElement(newTE, _part));

    showView();
  }
}

void KdeMasterEditor::addMeter() {
  pos();
  meter();
  Part * _part = part();
  if ((newMeter0>0) && (newMeter1>0) && (newPos>=0)) {
    if (index!=-1) {
      int k = 0;
      Event * ev = 0;
      for (Iterator i = Iterator(_part); !i.done() && k<=index; i++) {
	ev = (Event*) *i;
	k++;
      }
      if (ev!=0) sonG->doo(new RemoveElement(ev, _part));
    }
    MasterEvent * newME = new MasterEvent(newPos,newMeter0,newMeter1);
    sonG->doo(new AddElement(newME, _part));

    showView();
  }
}

void KdeMasterEditor::remove() {
  index = view->currentItem();
  if (index>=0) {
    int k = 0;
    Event * ev = 0;
    Part * _part = part();
    for (Iterator i = Iterator(_part); !i.done() && k<=index; i++) {
      ev = (Event*) *i;
      // if (ev!=0 && ev->isA()==MASTEREVENT)
      k++;
    }
    cout << "remove index " << index << ", k: " << k << endl;
    if (ev!=0) {
      if (ev->isA()!=MASTEREVENT) printf("PANIC KDEMASTEREDITOR");
      else sonG->doo(new RemoveElement(ev, _part));
    }
    showView();
  }
}

void KdeMasterEditor::exit() {
  delete this;
}

void KdeMasterEditor::insertTE(Event * te) {
  char * x = new char[40];
  Position pos = part()->start(te);
  int a = 0; int b = 0; int c = 0;
  sonG->bbt(a,b,c,pos);
  if (te->isA()==MASTEREVENT) {
    if (((MasterEvent*)te)->tempo()==0) { // meter
      snprintf(x,30," %d. %d. %d :      %d / %d",a,b,c,((MasterEvent*)te)->meter0(),((MasterEvent*)te)->meter1());
    } else { // tempo
      snprintf(x,30," %d. %d. %d : %d",a,b,c,((MasterEvent*)te)->tempo());
    }
  } else {
    snprintf(x,30," [%s]", te->ctype());
  }
  view->insertItem(x);
  delete[] x;
}

void KdeMasterEditor::showView() {
  view->clear();
  Event * ev = 0;
  Part * _part = part();
  for (Iterator i = Iterator(_part); !i.done(); i++) {
    ev = (Event*) *i;
    // if (ev!=0 && ev->isA()==MASTEREVENT) insertTE( (MasterEvent*) ev);
    insertTE(ev);
  }
  view->clearSelection();
  index = -1;
  update();
}

int KdeMasterEditor::Pitch(int i) {}



void KdeMasterEditor::update() {
  // cout << "update" << endl;
  _content->repaint(FALSE);
}

void KdeMasterEditor::setCursorPosition(long ticks) {
  // cout << "pos: " << ticks << endl;
  _content->moveCursor(ticks);
}

void KdeMasterEditor::gui(GuiEvents ev, bool flag) {
  switch (ev) {
  case CHANGES:
    break;
  case UNDO:
    _toolbar->setItemEnabled( ID_EDIT_UNDO, flag );
    _editmenu->setItemEnabled( ID_EDIT_UNDO, flag );
    break;
  case REDO:
    _toolbar->setItemEnabled( ID_EDIT_REDO, flag );
    _editmenu->setItemEnabled( ID_EDIT_REDO, flag );
    break;
  case SELECTIONS:
    _toolbar->setItemEnabled( ID_EDIT_CUT, flag );
    _toolbar->setItemEnabled( ID_EDIT_COPY, flag );
    _toolbar->setItemEnabled( ID_EDIT_CLEAR, flag );
    _toolbar->setItemEnabled( ID_EDIT_TRANSPOSE, flag );
    _editmenu->setItemEnabled( ID_EDIT_CUT, flag );
    _editmenu->setItemEnabled( ID_EDIT_COPY, flag );
    _editmenu->setItemEnabled( ID_EDIT_CLEAR, flag );
    _editmenu->setItemEnabled( ID_EDIT_TRANSPOSE, flag );
    _editmenu->setItemEnabled( ID_EDIT_QUANTIZE_ALL, flag );
    _editmenu->setItemEnabled( ID_EDIT_QUANTIZE_LENGTH, flag );
    _editmenu->setItemEnabled( ID_EDIT_FIXED_LENGTH, flag );
    break;
  case MEMORY:
    _toolbar->setItemEnabled( ID_EDIT_PASTE, flag );
    _editmenu->setItemEnabled( ID_EDIT_PASTE, flag );

    break;
  }
}

void KdeMasterEditor::print() {
}

void KdeMasterEditor::settings() {
}

void KdeMasterEditor::slotMenu(int i) {
  // cout << i << endl;
  _toolbar->commandCallback(i);
}


void KdeMasterEditor::slotViewMenu(int i) {
  bool ch = _viewmenu->isItemChecked(i);
  ch = !ch;
  _viewmenu->setItemChecked(i,ch);
  switch(i) {
  case ID_VIEW_TOOLBAR:
    if (ch) _toolbar->show(); else _toolbar->hide();
    break;
  case ID_VIEW_NOTEBAR:
    //if (ch) _notebar->show(); else _notebar->hide();
    break;
  case ID_VIEW_BUTTONBAR:
    //if (ch) _buttonbar->show(); else _buttonbar->hide();
    break;
  case ID_VIEW_SLIDER:
    if (ch) _scrollbar->show(); else _scrollbar->hide();
    break;
  }
}

void KdeMasterEditor::keyPressEvent ( QKeyEvent * ) {
  // _content->keyPressEvent(key);
}

void KdeMasterEditor::keyReleaseEvent ( QKeyEvent * ) {
  // _content->keyReleaseEvent(key);
}

#endif
#include "kdeMasterEditor.moc"
