/***************************************************************************
                          piano.cpp  -  description
                             -------------------
    begin                : Sat Jun 8 2002
    copyright            : (C) 2002 by Morten Hustveit
    email                : mortehu@ifi.uio.no
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qpainter.h>

#include "piano.h"

Piano::Piano(QWidget *parent, const char *name ) : QWidget(parent,name)
{
  setMinimumHeight(32);
  setMaximumHeight(32);
  setWFlags(Qt::WResizeNoErase);
  setWFlags(Qt::WRepaintNoErase);

  offset = 0;
  for(uint i = 0; i < 128; ++i)
    notes[i] = false;

  f_width = 9;
  s_width = 5;
  s_height = 18;
}

Piano::~Piano()
{
}

void Piano::resizeEvent(QResizeEvent* /*event*/)
{
  int max_x = (128 / 12 + 1) * 7 * 9;

  if(offset > max_x - width())
  {
    if(width() < max_x)
      offset = max_x - width();
    else
      offset = 0;

    update(); // use scroll() instead
  }
}

void Piano::updateKey(int note)
{
  int octave = note / 12;
  int key = note % 12;

  int x = octave * 7 * f_width;

  if(((key % 2) == 0) == (key < 5)) // White key
  {
    x += (key + 1) / 2 * f_width;

    update(x - offset, 0, f_width, height());
  }
  else // Black key
  {
//    1 3 6 8 10
    x += (key + 2) / 2 * f_width;

    update(x - s_width / 2 - offset, 0, s_width, s_height);
  }
}

void Piano::paintEvent(QPaintEvent* event)
{
  QPainter painter(this);

  int x = -offset;

  // Draw white keys first
  for(int i = 0; x < event->rect().right(); ++i)
  {
    int key = i % 12;

    if(((key % 2) == 0) != (key < 5))
      continue;

    if(x + f_width >= event->rect().left())
    {
      painter.setBrush(isSet(i) ? QColor(0x7FCFFF) : Qt::white);
      painter.setPen(Qt::black);
      painter.drawRect(x, 0, f_width + 1, height());

      painter.setPen(QColor(0x7F7F7F));
      painter.drawLine(x + 1, 1, x + 1, height() - 2);
      painter.drawLine(x + 2, height() - 2, x + f_width - 1, height() - 2);

      painter.setPen(QColor(0xCFCFCF));
      painter.drawLine(x + 2, 1, x + f_width - 1, 1);
      painter.drawLine(x + f_width - 1, 2, x + f_width - 1, height() - 3);
    }

    x += f_width;
  }
  
  x = -offset;

  // Draw black keys second (because the overlap the white)
  for(int i = 0; x - s_width / 2 < (int) event->rect().right(); ++i)
  {
    int key = i % 12;

    if(((key % 2) == 0) == (key < 5))
    {
      x += f_width;

      continue;
    }

    if(x + f_width - s_width / 2 >= event->rect().left())
    {
      painter.setBrush(isSet(i) ? QColor(0x6F7FFF) : QColor(0x3F3F3F));

      painter.setPen(Qt::black);
      painter.drawRect(x - s_width / 2, 0, s_width + 1, s_height);
      
      painter.setPen(QColor(0x7F7F7F));
      painter.drawLine(x - s_width / 2, 1, x - s_width / 2, s_height - 2);
    }
  }
}

void Piano::mousePressEvent(QMouseEvent* event)
{
  lastPos = event->pos();
}

void Piano::mouseReleaseEvent(QMouseEvent* /*event*/)
{
}

void Piano::mouseMoveEvent(QMouseEvent* event)
{
  if(event->state() & Qt::LeftButton)
  {
    int lastOffset = offset;

    int deltaX = event->pos().x() - lastPos.x();
    
    if(offset > deltaX)
      offset -= deltaX;
    else
      offset = 0;

    int max_x = (128 / 12 + 1) * 7 * 9;

    if(offset > max_x - width())
    {
      if(width() < max_x)
        offset = max_x - width();
      else
        offset = 0;
    }

    if(offset != lastOffset)
      update();

    lastPos = event->pos();
  }
}

void Piano::keyPressed(uint note)
{
  if(note <= 127)
    notes[note] = true;

  updateKey(note);
}

void Piano::keyReleased(uint note)
{
  if(note <= 127)
    notes[note] = false;

  updateKey(note);
}

void Piano::allReleased()
{
  for(uint i = 0; i < 128; ++i)
    notes[i] = false;

  update();
}

bool Piano::isSet(int note)
{
  if(note > 127)
    return false;

  return notes[note];
}

