/***************************************************************************
                          scheduler.h  -  description
                             -------------------
    begin                : Mon Jun 10 2002
    copyright            : (C) 2002 by Morten Hustveit
    email                : mortehu@ifi.uio.no
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SCHEDULER_H
#define SCHEDULER_H

#include <qthread.h>
#include <qevent.h>

/**
  *@author Morten Hustveit
  */

class SanetrackerApp;
class Configuration;

class MIDIMessageEvent : public QEvent
{
public:

  MIDIMessageEvent(uint status, uint data0, uint data1 = 0) : QEvent(User)
  {
    _status = status;
    _data[0] = data0;
    _data[1] = data1;
  }

  uint status() { return _status; }
  uint data(uint index = 0) { return _data[index]; }

protected:

  uint _status;
  uint _data[2];
};

class PositionEvent : public QEvent
{
public:

  PositionEvent() : QEvent(static_cast<QEvent::Type>(User + 1)) { }
};

class Scheduler : public QThread
{
public:

  Scheduler(SanetrackerApp* parent, const Configuration& config);
  ~Scheduler();

  void run();
  void stop(); // Stops the thread

  void play(uint measure = 0, uint measureTick = 0);
  void stopPlayback();

  uint measure() const { return _measure; }
  bool isPlaying() const { return playing; }
  bool isStopped() const { return stopped; }

  void setPatch(uint device, uint patch);

protected:

  SanetrackerApp* parent;

  uint tick;
  uint _measure;
  uint measureTick;
  bool playing;
  bool stopped;
  bool wasPlaying;

  QMap< uint, QMap<uint, uint> > notes;
  QMap< uint, uint > patches;

  const Configuration& _config;
};

#endif

